@@ -1,27 +1,31 @@
-### R code from vignette source 'article.Rnw'
-### Encoding: UTF-8
-
-###################################################
-### code chunk number 1: article.Rnw:1-7
-###################################################
+## ----echo=FALSE, results='hide'-----------------------------------------------
 # This should be set to FALSE in case you want to redo the sampling as well.
 # Default is TRUE to save CRAN some time.
 usePreCalcResults <- TRUE
 
 # Default here is FALSE because the object created is rather large
-usePreCalcResultsExtra <- FALSE 
+usePreCalcResultsExtra <- FALSE
 
+## ----setup, include=FALSE, cache=FALSE------------------------------
+knitr::render_sweave()
+knitr::opts_chunk$set(prompt = TRUE,
+                      fig.show = "hide",
+                      fig.keep = "high",
+                      warning = FALSE,
+                      error = FALSE,
+                      dev = c("png", "pdf"),
+                      dpi = 90,
+                      message = FALSE,
+                      echo = FALSE,
+                      eval=TRUE,
+                      #cache = TRUE,
+                      fig.path = "Figures/jss2014-",
+                      tidy = FALSE)
+base::options(continue = "+  ", prompt = "R> ", width = 70,
+  useFancyQuotes = FALSE)
+plotevery <- 1L # can be used for trace plots to make vignette smaller
 
-###################################################
-### code chunk number 2: preliminaries
-###################################################
-options(prompt = 'R> ', continue = '+  ')
-plotevery <- 10L # can be used for trace plots to make vignette smaller
-
-
-###################################################
-### code chunk number 3: usd1
-###################################################
+## ----usd1, echo=TRUE, fig.width=11, fig.height=6--------------------
 set.seed(123)
 library("stochvol")
 data("exrates")
@@ -31,121 +35,72 @@ plot(exrates$date, exrates$USD, type = "l",
   main = "Price of 1 EUR in USD")
 plot(exrates$date[-1], ret, type = "l", main = "Demeaned log returns")
 
-
-###################################################
-### code chunk number 4: usd2
-###################################################
+## ----usd2, echo=TRUE, fig.width=11, fig.height=6--------------------
 sim <- svsim(500, mu = -9, phi = 0.99, sigma = 0.1)
 par(mfrow = c(2, 1))
 plot(sim)
 
+## ----svsample, eval=TRUE, echo=TRUE---------------------------------
+ret <- ret[1:200]
+res <- svsample(ret, priormu = c(-10, 1), priorphi = c(20, 1.1),
+  priorsigma = 0.1, thin = 10)
 
-###################################################
-### code chunk number 5: article.Rnw:240-242 (eval = FALSE)
-###################################################
-## res <- svsample(ret, priormu = c(-10, 1), priorphi = c(20, 1.1),
-##   priorsigma = 0.1)
-
-
-###################################################
-### code chunk number 6: article.Rnw:245-251
-###################################################
-if (usePreCalcResultsExtra) {
- load("vignette_sampling_draws1.RData")
-} else {
- res <- svsample(ret, priormu = c(-10, 1), priorphi = c(20, 1.1), priorsigma = .1)
- #save(res, file = "vignette_sampling_draws1.RData")
-}
-
-
-###################################################
-### code chunk number 7: article.Rnw:273-274
-###################################################
+## ----summary, echo=TRUE---------------------------------------------
 summary(res, showlatent = FALSE)
 
+## ----paropar, include=FALSE-----------------------------------------
+par(mfrow = c(1, 1))
 
-###################################################
-### code chunk number 8: usd3
-###################################################
-volplot(res, forecast = 100, dates = exrates$date[-1])
+## ----usd3, echo=TRUE, fig.width=11, fig.height=7--------------------
+volplot(res, forecast = 100, dates = exrates$date[seq_along(ret)])
 
-
-###################################################
-### code chunk number 9: usd4
-###################################################
+## ----usd4, echo=TRUE, fig.width=11, fig.height=7--------------------
 res <- updatesummary(res, quantiles = c(0.01, 0.1, 0.5, 0.9, 0.99))
-volplot(res, forecast = 100, dates = exrates$date[-1])
+volplot(res, forecast = 100, dates = exrates$date[seq_along(ret)])
 
-
-###################################################
-### code chunk number 10: usd5
-###################################################
+## ----usd5, echo=TRUE, fig.width=8, fig.height=5---------------------
 par(mfrow = c(3, 1))
 paratraceplot(res)
 
-
-###################################################
-### code chunk number 11: usd6
-###################################################
+## ----usd6, echo=TRUE, fig.width=8, fig.height=3.5-------------------
 par(mfrow = c(1, 3))
 paradensplot(res, showobs = FALSE)
 
-
-###################################################
-### code chunk number 12: usd7
-###################################################
+## ----usd7, echo=TRUE, fig.width=9, fig.height=7---------------------
 plot(res, showobs = FALSE)
 
-
-###################################################
-### code chunk number 13: usd8
-###################################################
+## ----usd8, echo=TRUE, fig.width=9, fig.height=7---------------------
 myresid <- resid(res)
 plot(myresid, ret)
 
-
-###################################################
-### code chunk number 14: article.Rnw:405-411
-###################################################
+## ----beta1, echo=TRUE-----------------------------------------------
 set.seed(123456)
-n <- 1000
+n <- 200
 beta.true <- c(0.1, 0.5)
 sigma.true <- 0.01
 X <- matrix(c(rep(1, n), rnorm(n, sd = sigma.true)), nrow = n)
 y <- rnorm(n, X %*% beta.true, sigma.true)
 
-
-###################################################
-### code chunk number 15: article.Rnw:415-421
-###################################################
+## ----betaprior, echo=TRUE-------------------------------------------
 burnin <- 100
-draws <- 5000
+draws <- 1000
 b0 <- matrix(c(0, 0), nrow = ncol(X))
 B0inv <- diag(c(10^-10, 10^-10))
 c0 <- 0.001
 C0 <- 0.001
 
-
-###################################################
-### code chunk number 16: article.Rnw:424-428
-###################################################
+## ----betaprecalc, echo=TRUE-----------------------------------------
 p <- ncol(X)
 preCov <- solve(crossprod(X) + B0inv)
 preMean <- preCov %*% (crossprod(X, y) + B0inv %*% b0)
 preDf <- c0 + n/2 + p/2
 
-
-###################################################
-### code chunk number 17: article.Rnw:431-434
-###################################################
+## ----echo=TRUE------------------------------------------------------
 draws1 <- matrix(NA_real_, nrow = draws, ncol = p + 1)
 colnames(draws1) <- c(paste("beta", 0:(p-1), sep = "_"), "sigma")
 sigma2draw <- 1
 
-
-###################################################
-### code chunk number 18: article.Rnw:437-445
-###################################################
+## ----betaloop, echo=TRUE--------------------------------------------
 for (i in -(burnin-1):draws) {
   betadraw <- as.numeric(mvtnorm::rmvnorm(1, preMean,
     sigma2draw * preCov))
@@ -155,84 +110,60 @@ for (i in -(burnin-1):draws) {
   if (i > 0) draws1[i, ] <- c(betadraw, sqrt(sigma2draw))
 }
 
-
-###################################################
-### code chunk number 19: article.Rnw:448-449
-###################################################
+## ----colmeansbeta, echo=TRUE----------------------------------------
 colMeans(draws1)
 
+## ----draws1, eval=TRUE, echo=TRUE, fig.width=7.5, fig.height=6------
+plot(coda::mcmc(draws1))
 
-###################################################
-### code chunk number 20: homo
-###################################################
-par(mar = c(3.1, 1.8, 1.9, .5), mgp = c(1.8, .6, 0))
-plot(coda::mcmc(draws1[seq(1L, nrow(draws1), by = plotevery),]), show.obs = FALSE)
-
-
-###################################################
-### code chunk number 21: article.Rnw:459-460 (eval = FALSE)
-###################################################
-## plot(coda::mcmc(draws1), show.obs = FALSE)
-
-
-###################################################
-### code chunk number 22: article.Rnw:482-487
-###################################################
+## ----echo=TRUE------------------------------------------------------
 mu.true <- log(sigma.true^2)
 phi.true <- 0.97
 vv.true <- 0.3
 simresid <- svsim(n, mu = mu.true, phi = phi.true, sigma = vv.true)
 y <- X %*% beta.true + simresid$y
 
-
-###################################################
-### code chunk number 23: article.Rnw:490-496
-###################################################
-draws <- 50000
-burnin <- 1000
+## ----echo=TRUE------------------------------------------------------
+draws <- 5000
+burnin <- 500
 thinning <- 10
-priormu <- c(-10, 2)
-priorphi <- c(20, 1.5)
-priorsigma <- 1
+priors <- specify_priors(
+  mu = sv_normal(-10, 2),
+  phi = sv_beta(20, 1.5),
+  sigma2 = sv_gamma(0.5, 0.5)
+)
 
-
-###################################################
-### code chunk number 24: article.Rnw:499-506
-###################################################
+## ----echo=TRUE------------------------------------------------------
 draws2 <- matrix(NA_real_, nrow = floor(draws / thinning),
   ncol = 3 + n + p)
 colnames(draws2) <- c("mu", "phi", "sigma",
   paste("beta", 0:(p-1), sep = "_"), paste("h", 1:n, sep = "_"))
 betadraw <- c(0, 0)
-svdraw <- list(para = c(mu = -10, phi = 0.9, sigma = 0.2),
-  latent = rep(-10, n))
+paradraw <- list(mu = -10, phi = 0.9, sigma = 0.2)
+latentdraw <- rep(-10, n)
+paranames <- names(paradraw)
 
+## ----eval=FALSE, echo=TRUE------------------------------------------
+#  for (i in -(burnin-1):draws) {
+#    ytilde <- y - X %*% betadraw
+#    svdraw <- svsample_fast_cpp(ytilde, startpara = paradraw,
+#      startlatent = latentdraw, priorspec = priors)
+#    paradraw <- svdraw$para
+#    latentdraw <- drop(svdraw$latent)
+#    normalizer <- as.numeric(exp(-latentdraw / 2))
+#    Xnew <- X * normalizer
+#    ynew <- y * normalizer
+#    Sigma <- solve(crossprod(Xnew) + B0inv)
+#    mu <- Sigma %*% (crossprod(Xnew, ynew) + B0inv %*% b0)
+#    betadraw <- as.numeric(mvtnorm::rmvnorm(1, mu, Sigma))
+#    if (i > 0 && i %% thinning == 0) {
+#      draws2[i/thinning, 1:3] <- drop(paradraw)[paranames]
+#      draws2[i/thinning, 4:5] <- betadraw
+#      draws2[i/thinning, 6:(n+5)] <- latentdraw
+#    }
+#  }
 
-###################################################
-### code chunk number 25: article.Rnw:513-530 (eval = FALSE)
-###################################################
-## for (i in -(burnin-1):draws) {
-##   ytilde <- y - X %*% betadraw
-##   svdraw <- svsample2(ytilde, startpara = para(svdraw),
-##     startlatent = latent(svdraw), priormu = priormu,
-##     priorphi = priorphi, priorsigma = priorsigma)
-##   normalizer <- as.numeric(exp(-latent(svdraw) / 2))
-##   Xnew <- X * normalizer
-##   ynew <- y * normalizer
-##   Sigma <- solve(crossprod(Xnew) + B0inv)
-##   mu <- Sigma %*% (crossprod(Xnew, ynew) + B0inv %*% b0)
-##   betadraw <- as.numeric(mvtnorm::rmvnorm(1, mu, Sigma))
-##   if (i > 0 & i %% thinning == 0) { 
-##     draws2[i/thinning, 1:3] <- para(svdraw)
-##     draws2[i/thinning, 4:5] <- betadraw
-##     draws2[i/thinning, 6:(n+5)] <- latent(svdraw)
-##   }
-## }
-
-
-###################################################
-### code chunk number 26: article.Rnw:533-562
-###################################################
+## ----longerbeta-----------------------------------------------------
 if (usePreCalcResults) {
  load("vignette_sampling_draws2.RData")
 } else {
@@ -240,12 +171,13 @@ for (i in -(burnin-1):draws) {
 
  # draw latent volatilities and AR-parameters:
  ytilde <- y - X %*% betadraw
- svdraw <- svsample2(ytilde, startpara = para(svdraw),
-                     startlatent = latent(svdraw), priormu = priormu,
-                     priorphi = priorphi, priorsigma = priorsigma)
+ svdraw <- svsample_fast_cpp(ytilde, startpara = paradraw,
+   startlatent = latentdraw, priorspec = priors)
+ paradraw <- svdraw$para
+ latentdraw <- drop(svdraw$latent)
 
  # draw the betas:
- normalizer <- as.numeric(exp(-latent(svdraw)/2))
+ normalizer <- as.numeric(exp(-latentdraw/2))
  Xnew <- X * normalizer
  ynew <- y * normalizer
  Sigma <- solve(crossprod(Xnew) + B0inv)
@@ -253,45 +185,30 @@ for (i in -(burnin-1):draws) {
  betadraw <- as.numeric(mvtnorm::rmvnorm(1, mu, Sigma))
 
  # store the results:
- if (i > 0 & i %% thinning == 0) { 
-  draws2[i/thinning, 1:3] <- para(svdraw)
+ if (i > 0 & i %% thinning == 0) {
+  draws2[i/thinning, 1:3] <- drop(paradraw)[paranames]
   draws2[i/thinning, 4:5] <- betadraw
-  draws2[i/thinning, 6:(n+5)] <- latent(svdraw)
+  draws2[i/thinning, 6:(n+5)] <- latentdraw
  }
 }
  draws2selection <- draws2[,4:8]
- #save(draws2selection, file = 'vignette_sampling_draws2.RData')
+ save(draws2selection, file = 'vignette_sampling_draws2.RData')
 }
 
+## ----betaplot2, echo=TRUE, eval=FALSE-------------------------------
+#  plot(coda::mcmc(draws2[, 4:8]))
 
-###################################################
-### code chunk number 27: article.Rnw:566-567 (eval = FALSE)
-###################################################
-## plot(coda::mcmc(draws2[, 4:7]), show.obs = FALSE)
-
-
-###################################################
-### code chunk number 28: hetero
-###################################################
+## ----hetero, fig.width=7.5, fig.height=8----------------------------
 par(mar = c(3.1, 1.8, 1.9, .5), mgp = c(1.8, .6, 0))
-plot(coda::mcmc(draws2selection[seq(1L, nrow(draws2selection), by = plotevery), 1:4]), show.obs = FALSE)
+plot(coda::mcmc(draws2selection[seq(1L, nrow(draws2selection), by = plotevery), 1:4]))
 
+## ----colmeans2, eval=FALSE, echo=TRUE-------------------------------
+#  colMeans(draws2[, 4:8])
 
-###################################################
-### code chunk number 29: article.Rnw:585-586 (eval = FALSE)
-###################################################
-## colMeans(draws2[, 4:8])
-
-
-###################################################
-### code chunk number 30: article.Rnw:589-590
-###################################################
+## ----echo=FALSE, eval=TRUE------------------------------------------
 colMeans(draws2selection)
 
-
-###################################################
-### code chunk number 31: scatter
-###################################################
+## ----scatter, echo=TRUE, fig.width=10.2-----------------------------
 x <- log(exrates$USD[-length(exrates$USD)])
 y <- log(exrates$USD[-1])
 X <- matrix(c(rep(1, length(x)), x), nrow = length(x))
@@ -301,10 +218,7 @@ plot(x,y, xlab=expression(log(p[t])), ylab=expression(log(p[t+1])),
     col="#00000022", pch=16, cex=1)
 abline(0,1)
 
-
-###################################################
-### code chunk number 32: article.Rnw:655-876
-###################################################
+## ----results='hide'-------------------------------------------------
 if (usePreCalcResults) {
  load("vignette_sampling_realworld.RData")
 } else {
@@ -334,6 +248,13 @@ C0 <- 0.001
 
 B0inv <- solve(B0)
 
+# specify priors in an object
+priors <- specify_priors(
+  mu = sv_normal(-10, 2),
+  phi = sv_beta(20, 1.5),
+  sigma2 = sv_gamma(0.5, 0.5)
+)
+
 ## initialize some space to hold results:
 realres <- vector("list", 3)
 
@@ -349,8 +270,9 @@ latents <- 3+p+(1:n)
 
 ## starting values:
 betadraw <- rep(.1, p)
-svdraw <- list(para = c(mu = -10, phi = .8, sigma = .2),
-	       latent = rep(-10, n))
+paradraw <- list(mu = -10, phi = 0.8, sigma = 0.2)
+latentdraw <- rep(-10, n)
+paranames <- names(paradraw)
 
 ## sampler:
 tim <- system.time(
@@ -358,12 +280,13 @@ for (i in -(burnin-1):draws) {
  if (i%%1000 == 0) cat("Iteration", i, "done.\n")
  # draw latent volatilities and SV-parameters:
  ytilde <- y - X %*% betadraw
- svdraw <- svsample2(ytilde, startpara=para(svdraw),
-		      startlatent=latent(svdraw), priormu=priormu,
-		      priorphi=priorphi, priorsigma=priorsigma)
+ svdraw <- svsample_fast_cpp(ytilde, startpara = paradraw,
+  startlatent = latentdraw, priorspec = priors)
+ paradraw <- svdraw$para
+ latentdraw <- drop(svdraw$latent)
 
  # draw the betas:
- normalizer <- as.numeric(exp(-latent(svdraw)/2))
+ normalizer <- as.numeric(exp(-latentdraw/2))
  Xnew <- X * normalizer
  ynew <- y * normalizer
  Sigma <- solve(crossprod(Xnew) + B0inv)
@@ -371,9 +294,9 @@ for (i in -(burnin-1):draws) {
  betadraw <- as.numeric(mvtnorm::rmvnorm(1, mu, Sigma))
 
  # store the results:
- if (i > 0 & i %% thinning == 0) { 
-  realres[[1]][i/thinning, paras] <- para(svdraw)
-  realres[[1]][i/thinning, latents] <- latent(svdraw)
+ if (i > 0 & i %% thinning == 0) {
+  realres[[1]][i/thinning, paras] <- drop(paradraw)[paranames]
+  realres[[1]][i/thinning, latents] <- latentdraw
   realres[[1]][i/thinning, betas] <- betadraw
  }
 }
@@ -399,14 +322,14 @@ for (i in -(burnin-1):draws) {
  if (i%%1000 == 0) cat("Iteration", i, "done.\n")
  # draw beta:
  betadraw <- as.numeric(mvtnorm::rmvnorm(1, preMean, sigma2draw*preCov))
- 
+
  # draw sigma^2:
  tmp <- C0 + .5*(crossprod(y - X%*%betadraw) +
 		crossprod((betadraw-b0), B0inv) %*% (betadraw-b0))
  sigma2draw <- 1/rgamma(1, preDf, rate=tmp)
 
  # store the results:
- if (i > 0 & i %% thinning == 0) { 
+ if (i > 0 & i %% thinning == 0) {
   realres[[2]][i/thinning, 1:p] <- betadraw
   realres[[2]][i/thinning, p+1] <- sqrt(sigma2draw)
  }
@@ -428,7 +351,7 @@ s2s <- function(y, para, y20, s20) {
  s2 <- rep(NA_real_, length(y))
  s2[1] <- para[1] + para[2]*y20 + para[3]*s20
  for (i in 2:length(y)) s2[i] <- para[1] + para[2]*y[i-1]^2 + para[3]*s2[i-1]
- s2	
+ s2
 }
 
 ## This is a very simple (and slow) random-walk MH sampler
@@ -462,7 +385,7 @@ betadraw <- as.numeric(coef(tmplm))
 
 paradraw <- startvals
 paraprop <- rep(NA_real_, 3)
- 
+
 s2draw <- rep(.1, length(y))
 s2prop <- rep(NA_real_, length(y))
 
@@ -471,9 +394,9 @@ accepts <- 0
 ## sampler:
 tim3 <- system.time(
 for (i in -(burnin-1):draws) {
- 
+
  if (i%%1000 == 0) cat("Iteration", i, "done.\n")
- 
+
  # draw volatilities and GARCH-parameters:
  ytilde <- y - X %*% betadraw  # regression residuals
 
@@ -498,7 +421,7 @@ for (i in -(burnin-1):draws) {
  betadraw <- as.numeric(mvtnorm::rmvnorm(1, mu, Sigma))
 
  # store the results:
- if (i > 0 & i %% thinning == 0) { 
+ if (i > 0 & i %% thinning == 0) {
   realres[[3]][i/thinning, paras] <- paradraw
   realres[[3]][i/thinning, latents] <- sqrt(s2draw)
   realres[[3]][i/thinning, betas] <- betadraw
@@ -524,13 +447,10 @@ realresselection <- vector("list", 3)
 realresselection[[1]] <- realres[[1]][,c('beta_0', 'beta_1')]
 realresselection[[2]] <- realres[[2]][,c('beta_0', 'beta_1')]
 realresselection[[3]] <- realres[[3]][,c('beta_0', 'beta_1')]
-#save(realresselection, standresidmeans, standresidmeans2, standresidmeans3, file = 'vignette_sampling_realworld.RData')
+save(realresselection, standresidmeans, standresidmeans2, standresidmeans3, file = 'vignette_sampling_realworld.RData')
 }
 
-
-###################################################
-### code chunk number 33: betapost
-###################################################
+## ----betapost, fig.width=8.7, fig.height=7--------------------------
 smootherfactor <- 1.8
 par(mar = c(2.9, 2.9, 2.7, .5), mgp = c(1.8,.6,0), tcl = -.4)
 layout(matrix(c(1,2,3,3), byrow=TRUE, nrow=2))
@@ -573,10 +493,7 @@ plot(beta0, beta1, col=cols, pch=pchs,
     main="Scatterplot of posterior draws")
 legend("topright", c("SV", "GARCH", "homosked."), col=c(1,4,2), pch=1:3)
 
-
-###################################################
-### code chunk number 34: qqplot
-###################################################
+## ----qqplot, fig.width=8, fig.height=7------------------------------
 par(mfrow=c(3,2), mar = c(3.1, 3.3, 2.0, .5), mgp = c(1.7,.5,0),
     tcl = -.4)
 plot(standresidmeans2, main="Residual scatterplot (homoskedastic errors)",
@@ -598,4 +515,3 @@ qqplot(qnorm(ppoints(length(standresidmeans)), mean=0, sd=1),
        xlab = "Theoretical N(0,1)-quantiles", ylab = "Empirical Quantiles")
 abline(0,1)
 
-
